// For general information on the Pynini grammar compilation library, see
// pynini.opengrm.org.

#include "pynini_replace.h"

namespace fst {
namespace script {

void PyniniReplace(const FstClass &root,
                   const std::vector<StringFstClassPair> &pairs,
                   MutableFstClass *ofst, const ReplaceOptions &opts) {
  if (!ArcTypesMatch(root, *ofst, "PyniniReplace")) {
    ofst->SetProperties(kError, kError);
    return;
  }
  for (auto it = pairs.cbegin(); it != pairs.cend(); ++it) {
    if (!ArcTypesMatch(*it->second, *ofst, "PyniniReplace")) {
      ofst->SetProperties(kError, kError);
      return;
    }
  }
  PyniniReplaceArgs args(root, pairs, ofst, opts);
  Apply<Operation<PyniniReplaceArgs>>("PyniniReplace", ofst->ArcType(), &args);
}

REGISTER_FST_OPERATION(PyniniReplace, StdArc, PyniniReplaceArgs);
REGISTER_FST_OPERATION(PyniniReplace, LogArc, PyniniReplaceArgs);
REGISTER_FST_OPERATION(PyniniReplace, Log64Arc, PyniniReplaceArgs);

void PyniniPdtReplace(const FstClass &root,
                      const std::vector<StringFstClassPair> &pairs,
                      MutableFstClass *ofst,
                      std::vector<LabelPair> *parens,
                      PdtParserType type) {
  if (!ArcTypesMatch(root, *ofst, "PyniniReplace")) {
    ofst->SetProperties(kError, kError);
    return;
  }
  for (auto it = pairs.cbegin(); it != pairs.cend(); ++it) {
    if (!ArcTypesMatch(*it->second, *ofst, "PyniniPdtReplace")) {
      ofst->SetProperties(kError, kError);
      return;
    }
  }
  PyniniPdtReplaceArgs args(root, pairs, ofst, parens, type);
  Apply<Operation<PyniniPdtReplaceArgs>>("PyniniPdtReplace", ofst->ArcType(),
                                         &args);
}

REGISTER_FST_OPERATION(PyniniPdtReplace, StdArc, PyniniPdtReplaceArgs);
REGISTER_FST_OPERATION(PyniniPdtReplace, LogArc, PyniniPdtReplaceArgs);
REGISTER_FST_OPERATION(PyniniPdtReplace, Log64Arc, PyniniPdtReplaceArgs);

}  // namespace script
}  // namespace fst

