// For general information on the Pynini grammar compilation library, see
// pynini.opengrm.org.

#ifndef PYNINI_STRINGIFY_H_
#define PYNINI_STRINGIFY_H_

// This file contains utility functions which allow conversion of kString
// FSTs to C++ strings; attempting to call these on a non-kString FST may
// produce jibberish; the reason(s) your FST is not kString will be printed
// to VLOG(2).

#include <fst/fstlib.h>
#include <fst/script/arg-packs.h>
#include <fst/script/fstscript.h>
#include "pathsscript.h"  // For TokenType, etc.

namespace fst {
namespace script {

typedef args::Package<const FstClass &, TokenType,
                      const SymbolTable *, string *> PyniniStringifyInnerArgs;

typedef args::WithReturnValue<bool, PyniniStringifyInnerArgs>
    PyniniStringifyArgs;

template <class Arc>
void PyniniStringify(PyniniStringifyArgs *args) {
  const Fst<Arc> &fst = *(args->args.arg1.GetFst<Arc>());
  StringPrinter<Arc> sprinter(GetStringPrinterTokenType<Arc>(args->args.arg2),
                              args->args.arg3);
  args->retval = sprinter(fst, args->args.arg4);
}

bool PyniniStringify(const FstClass &fst, TokenType token_type,
                     const SymbolTable *, string *);

}  // namespace script
}  // namespace fst

#endif  // PYNINI_STRINGIFY_H_

