// For general information on the Pynini grammar compilation library, see
// pynini.opengrm.org.

#include <fst/script/script-impl.h>
#include "stringcompilescript.h"

namespace fst {
namespace script {

bool CompileByteString(const string &str, const WeightClass &wc,
                       MutableFstClass *fst) {
  StringCompileInnerArgs iargs(str, wc, fst);
  StringCompileArgs args(iargs);
  Apply<Operation<StringCompileArgs>>("CompileByteString", fst->ArcType(),
                                      &args);
  return args.retval;
}

bool CompileUTF8String(const string &str, const WeightClass &wc,
                       MutableFstClass *fst) {
  StringCompileInnerArgs iargs(str, wc, fst);
  StringCompileArgs args(iargs);
  Apply<Operation<StringCompileArgs>>("CompileUTF8String", fst->ArcType(),
                                      &args);
  return args.retval;
}

bool CompileBracketedByteString(const string &str, const WeightClass &wc,
                                MutableFstClass *fst) {
  StringCompileInnerArgs iargs(str, wc, fst);
  StringCompileArgs args(iargs);
  Apply<Operation<StringCompileArgs>>("CompileBracketedByteString",
                                      fst->ArcType(), &args);
  return args.retval;
}

bool CompileBracketedUTF8String(const string &str, const WeightClass &wc,
                                MutableFstClass *fst) {
  StringCompileInnerArgs iargs(str, wc, fst);
  StringCompileArgs args(iargs);
  Apply<Operation<StringCompileArgs>>("CompileBracketedUTF8String",
                                      fst->ArcType(), &args);
  return args.retval;
}

REGISTER_FST_OPERATION(CompileByteString, StdArc, StringCompileArgs);
REGISTER_FST_OPERATION(CompileByteString, LogArc, StringCompileArgs);
REGISTER_FST_OPERATION(CompileByteString, Log64Arc, StringCompileArgs);

REGISTER_FST_OPERATION(CompileUTF8String, StdArc, StringCompileArgs);
REGISTER_FST_OPERATION(CompileUTF8String, LogArc, StringCompileArgs);
REGISTER_FST_OPERATION(CompileUTF8String, Log64Arc, StringCompileArgs);

REGISTER_FST_OPERATION(CompileBracketedByteString, StdArc, StringCompileArgs);
REGISTER_FST_OPERATION(CompileBracketedByteString, LogArc, StringCompileArgs);
REGISTER_FST_OPERATION(CompileBracketedByteString, Log64Arc, StringCompileArgs);

REGISTER_FST_OPERATION(CompileBracketedUTF8String, StdArc, StringCompileArgs);
REGISTER_FST_OPERATION(CompileBracketedUTF8String, LogArc, StringCompileArgs);
REGISTER_FST_OPERATION(CompileBracketedUTF8String, Log64Arc, StringCompileArgs);

}  // namespace script
}  // namespace fst

